-- This file loads the game module, and loads data
local KeyBind = require "engine.KeyBind"
local DamageType = require "engine.DamageType"
local ActorStats = require "engine.interface.ActorStats"
local ActorResource = require "engine.interface.ActorResource"
local ActorTalents = require "engine.interface.ActorTalents"
local ActorAI = require "engine.interface.ActorAI"
local ActorLevel = require "engine.interface.ActorLevel"
local ActorTemporaryEffects = require "engine.interface.ActorTemporaryEffects"
local ActorInventory = require "engine.interface.ActorInventory"
local Birther = require "engine.Birther"
local UIBase = require "engine.ui.Base"
local Quest = require "engine.Quest"

profile.mod.allow_build = profile.mod.allow_build or {}

UIBase.ui = "simple"

-- Additional entities resolvers
dofile("/mod/resolvers.lua")

-- Useful keybinds
KeyBind:load("move,hotkeys,inventory,actions,interface,debug")

-- Damage types
DamageType:loadDefinition("/data/damage_types.lua")

-- Talents
ActorTalents:loadDefinition("/data/talents/talents.lua")

-- Timed Effects
ActorTemporaryEffects:loadDefinition("/data/timed_effects.lua")

-- Actor resources
ActorResource:defineResource("Power", "power", nil, "power_regen", "Power represent your ability to use special talents.")
ActorResource:defineResource("Stamina", "stamina", nil, "stamina_regen", "Stamina represents your physical fatigue. Using abilities and being hit in combat reduces it.")

-- Actor stats
ActorStats:defineStat("Strength",	"str", 10, 1, 25, "Strength defines your character's ability to apply physical force. It increases your melee damage, damage with heavy weapons, your chance to resist physical effects, and carrying capacity.")
ActorStats:defineStat("Dexterity",	"dex", 10, 1, 25, "Dexterity defines your character's ability to be agile and alert. It increases your chance to hit, your ability to avoid attacks and your damage with light weapons.")
ActorStats:defineStat("Intelligence",	"int", 10, 1, 25, "Intelligence defines your character's metal abilities. It affects you skill in using devices.")
ActorStats:defineStat("Constitution",	"con", 10, 1, 25, "Constitution defines your character's ability to withstand and resist damage. It increases your maximum life and physical resistance.")

-- Actor inventory
ActorInventory:defineInventory("MAINHAND", "Main hand", true, "Most weapons are held in the main hand.")
ActorInventory:defineInventory("HEAD", "Head", true, "Hats and helmets go on the head.")
ActorInventory:defineInventory("BELT", "Ammo belt", true, "Holds gredanes.")
ActorInventory:defineInventory("CYBERWARE", "Cyberware", true, "Machinery connected to the CNS, most have activatable abilities.")

-- Actor AIs
ActorAI:loadDefinition("/engine/ai/")

-- Birther descriptor
Birther:loadDefinition("/data/birth/descriptors.lua")

return {require "mod.class.Game" }
